import { Component, EventEmitter, ViewChild } from "@angular/core";

import { ServiceFactory } from "../../services/service-factory";
import { InternationalisationService } from "../../services/internationalisation/internationalisation.service";

@Component({
    selector: "load-calc",
    templateUrl: "./load-calculator.component.html"
})
export class LoadCalculatorComponent {
    @ViewChild("fileSelector") fileSelector;

    private _selectFile: File;

    /**
     * liste des calculettes affichées. Forme des objets :
     * "title": nom de la calculette
     * "selected": flag de sélection pour la sauvegarde
     */
    private _calculators: any[];

    /**
     * événement émis lors du clic sur "annuler"/"charger"
     * utilisé par la promise de gestion de la confirmation/annulation de la sauvegarde
     */
    private _confirmResult = new EventEmitter();

    // services
    private intlService: InternationalisationService;

    constructor() {
        this.intlService = ServiceFactory.instance.internationalisationService;
    }

    public get uitextDialogTitle() {
        return "Charger des calculettes";
    }

    public get uitextCancel() {
        // return this.intlService.localizeText("INFO_OPTION_NO");
        return "Annuler";
    }

    public get uitextLoad() {
        // return this.intlService.localizeText("INFO_OPTION_NO");
        return "Charger";
    }

    public get uitextSelectAll() {
        return "Toutes";
    }

    public get uitextDeselectAll() {
        return "Aucune";
    }

    /**
     * calcule l'état du bouton charger
     */
    public get disableLoadButton() {
        // pas de fichier sélectionné -> bouton grisé
        if (this._selectFile === undefined) {
            return true;
        }

        // au moins une calculette sélectionnée -> dégrisé
        if (this._calculators !== undefined) {
            for (const c of this._calculators) {
                if (c.selected) {
                    return false;
                }
            }
        }

        // grisé sinon
        return true;
    }

    public run(): Promise<any[]> {
        // promise de gestion de la confirmation/annulation de la sauvegarde
        return new Promise((resolve, reject) => {
            this._confirmResult.subscribe((confirm) => {
                if (confirm) {
                    resolve(this._calculators);
                } else {
                    reject("canceled");
                }
            });
        });
    }

    public get showSelectButtons(): boolean {
        return this._calculators && this._calculators.length !== 0;
    }

    private getSelectedFile(): File {
        const files: { [key: string]: File } = this.fileSelector.nativeElement.files;
        for (const key in files) {
            if (!isNaN(Number(key))) {
                return files[key];
            }
        }
        return undefined;
    }

    public onFileSelect() {
        const formService = ServiceFactory.instance.formulaireService;
        this._selectFile = this.getSelectedFile();
        if (this._selectFile !== undefined) {
            formService.calculatorInfosFromSessionFile(this._selectFile).then(
                calcInfos => {
                    this._calculators = calcInfos;
                    for (const n of this._calculators) {
                        n["selected"] = true;
                    }
                });
        }
    }

    public get selectedFile(): File {
        return this._selectFile;
    }

    public get calculators(): any[] {
        return this._calculators;
    }

    private isSelected(c: any) {
        return c.selected ? "checked" : undefined;
    }

    private onCheckCalc(event: any) {
        for (const c of this._calculators) {
            if (c.uid === event.target.value) {
                c.selected = event.target.checked;
                break;
            }
        }
    }

    public selectAll() {
        for (const c of this._calculators) {
            c.selected = true;
        }
    }

    public deselectAll() {
        for (const c of this._calculators) {
            c.selected = false;
        }
    }

    private set confirmed(b: boolean) {
        setTimeout(() => {
            this._confirmResult.next(b);
        }, 0);
    }

    /**
     * appelé quand on clique sur annuler
     */
    public cancelLoad() {
        this.confirmed = false;
    }

    /**
     * appelé quand on clique sur charger
     */
    public confirmLoad() {
        this.confirmed = true;
    }
}
