import { Component, Input, ChangeDetectorRef } from "@angular/core";

import { GenericInputComponent } from "../generic-input/generic-input.component";
import { InternationalisationService } from "../../services/internationalisation/internationalisation.service";
import { NgParameter } from "../../formulaire/ngparam";
import { Message } from "jalhyd";

@Component({
    selector: "value-list",
    templateUrl: "../generic-input/generic-input.component.html"
})
export class ValueListComponent extends GenericInputComponent {
    constructor(private intlService: InternationalisationService, cdRef: ChangeDetectorRef) {
        super(cdRef);
    }

    /**
     * paramètre géré
     */
    private get _param(): NgParameter {
        return this._model;
    }

    protected getModelValue(): any {
        if (this._param === undefined || this._param === null) {
            return undefined;
        }
        return this._param.valueList;
    }

    protected setModelValue(sender: any, l: any) {
        if (typeof (l) === "number") {
            this._param.valueList = [];
            this._param.valueList.push(l);
        } else {
            this._param.valueList = l;
        }
    }

    protected validateModelValue(v: any): { isValid: boolean, message: string } {
        let msg;
        let valid = false;

        if (v instanceof Array) {
            valid = true;
            try {
                this._param.checkList(v);
            } catch (ex) {
                valid = false;
                if (ex instanceof Message) {
                    msg = this.intlService.localizeMessage(ex);
                } else {
                    msg = "invalid value";
                }
            }
        } else {
            msg = "Veuillez entrer une liste de nombres";
        }

        return { isValid: valid, message: msg };
    }

    protected modelToUI(v: any): string {
        let res = "";
        if (v !== undefined && v !== null) {
            for (const e of v) {
                if (res !== "") {
                    res += ";";
                }
                res += String(e);
            }
        }
        return res;
    }

    protected validateUIValue(ui: string): { isValid: boolean, message: string } {
        let valid = false;
        let msg: string;

        const tmp: string[] = ui.split(";");
        let res = true;
        for (const v of tmp) {
            const isnum = v !== "" && (+v === +v);
            res = res && isnum;
            if (!res) {
                break;
            }
        }

        if (!res) {
            msg = "Veuillez entrer une liste de nombres";
        } else {
            valid = true;
        }

        return { isValid: valid, message: msg };
    }

    protected uiToModel(ui: string) {
        const tmp: string[] = ui.split(";");
        const res = [];
        for (const v of tmp) {
            res.push(+v);
        }
        return res;
    }
}
