import { Component, Input } from "@angular/core";

import { ResultElement } from "jalhyd";

import { ApplicationSetupService } from "../../services/app-setup/app-setup.service";
import { InternationalisationService } from "../../services/internationalisation/internationalisation.service";
import { OnChanges } from "@angular/core/src/metadata/lifecycle_hooks";
import { ServiceFactory } from "../../services/service-factory";

/**
 * classe de base pour l'affichage d'un ResultElement
 * étendue par HorizontalResultElementComponent et VerticalResultElementComponent
 */
@Component({
    selector: "[result-element-base]",
    template: `<div></div>`
})
export class ResultElementBaseComponent implements OnChanges {
    /**
     * ResultElement à afficher
     */
    protected _resultElement: ResultElement;

    @Input("result-element")
    private set resultElement(re: ResultElement) {
        this._resultElement = re;
    }

    /**
     * valeur numérique du résultat
     */
    protected _value: string;

    /**
     * true si la valeur est en erreur
     */
    protected _hasError: boolean;

    /**
     * true si le result element a une valeur
     */
    protected _hasValue: boolean;

    /**
     * code HTML du tooltip
     */
    protected _htmlTooltip: string;

    /**
     * true si pas de texte à afficher dans le tooltip
     */
    protected _emptyTooltip = false;

    protected appSetupService: ApplicationSetupService;

    protected intlService: InternationalisationService;

    constructor(
    ) {
        this.appSetupService = ServiceFactory.instance.applicationSetupService;
        this.intlService = ServiceFactory.instance.internationalisationService;
    }

    /**
     * appelé quand les @Input changent
     */
    ngOnChanges() {
        this.updateTooltip();
    }

    private updateTooltip() {
        // valeur à afficher

        const nDigits = this.appSetupService.displayDigits;
        const r: ResultElement = this._resultElement;
        this._hasValue = r !== undefined && r.vCalc !== undefined;
        this._hasError = r === undefined || (r.vCalc === undefined && r.extraResults.length > 0);
        this._value = this._hasValue ? this._value = r.vCalc.toFixed(nDigits) : " ";

        // texte du tooltip

        let res = "";

        if (this._resultElement !== undefined) {
            for (const m of this._resultElement.log.messages) {
                if (res.length > 0) {
                    res += "<br/>";
                }
                res += this.intlService.localizeMessage(m);
            }
        }
        this._htmlTooltip = res;
        this._emptyTooltip = this._htmlTooltip.length === 0;
    }

    public get hasValue() {
        return this._hasValue;
    }

    private get resultValue(): string {
        return this._value;
    }

    public get hasError() {
        return this._hasError && !this._emptyTooltip;
    }

    private get htmlTooltip(): string {
        return this._htmlTooltip;
    }

    private get tooltipDisabled(): boolean {
        return this._emptyTooltip;
    }
}
