import { Nub, Result, ParamDomainValue } from "jalhyd";

import { FormResult } from "./form-result";
import { FormulaireDefinition } from "./form-definition";
import { NgParameter } from "../ngparam";

export abstract class FormCompute {
    constructor(protected _formBase: FormulaireDefinition, protected _formResult: FormResult) {
    }

    protected abstract compute();

    protected get formResult(): FormResult {
        return this._formResult;
    }

    /**
     * retourne un identifiant du paramètre dans le formulaire
     * surchargé dans le cas des ouvrages //
     */
    protected getParameterRefid(p: NgParameter) {
        return p.symbol;
    }

    /**
     * lance le calcul d'un paramètre en déterminant une valeur initiale
     */
    protected runNubCalc(nub: Nub, computedParam: NgParameter, prec: number): Result {
        let init: number;
        switch (computedParam.domain.domain) {
            case ParamDomainValue.ANY:
                if (computedParam && computedParam.isDefined) {
                    init = computedParam.getValue();
                }
                if (init === undefined) {
                    init = 0;
                }
                break;

            case ParamDomainValue.POS_NULL:
                if (computedParam && computedParam.isDefined) {
                    init = Math.max(computedParam.getValue(), 0);
                }
                if (init === undefined) {
                    init = 0;
                }
                break;

            case ParamDomainValue.INTERVAL:
                init = (computedParam.domain.minValue + computedParam.domain.maxValue) / 2;
                break;

            case ParamDomainValue.NOT_NULL:
                if (computedParam && computedParam.isDefined) {
                    init = computedParam.getValue();
                }
                if (init === undefined || init === 0) {
                    init = 1e-8;
                }
                break;

            case ParamDomainValue.POS:
                if (computedParam && computedParam.isDefined) {
                    init = Math.max(computedParam.getValue(), 1e-8);
                }
                if (init === undefined) {
                    init = 1e-8;
                }
                break;
        }

        return nub.CalcSerie(prec, init, this.getParameterRefid(computedParam));
    }

    public doCompute() {
        this._formResult.resetResults();

        this.compute();

        this._formBase.notifyObservers({
            "action": "resultsUpdated",
        }, this._formBase);
    }
}
